# -*- coding: utf-8 -*-
import os
from random import randint
from functools import partial
import json as jsonOut

if response.session_file:
    session.forget(response)

# Models

if request.function in ('db', 'queries', 'updates', 'fortune'):
    DBHOST = os.environ.get('DBHOST', 'localhost')
    DATABASE_URI = "mysql://benchmarkdbuser:benchmarkdbpass@%s:3306/hello_world" % DBHOST

    db = DAL(DATABASE_URI, migrate_enabled=False)

    if request.function == 'fortune':
        db.define_table('fortune',
            Field('message'))
    else:
        db.define_table('world',
            Field('randomNumber', 'integer'))

# Helpers

def getQueryNum(queryString):
    try:
        num_queries = int(queryString)
        if num_queries < 1:
            return 1
        if num_queries > 500:
            return 500
        return num_queries
    except ValueError:
         return 1

def processor(rows, fields, colnames, cacheable):
    colnames = [col.rpartition('.')[-1] for col in colnames]
    return [dict(zip(colnames, row)) for row in rows]

def get_world(wid):
    return db(db.world.id == wid).select(processor=processor)[0]

# Actions

def plaintext():
    response.headers["Content-Type"] = "text/plain"
    return "Hello, World!"

def json():
    response.headers["Content-Type"] = "application/json"
    return jsonOut.dumps({"message": "Hello, World!"})

def db():
    response.headers["Content-Type"]="application/json"
    return jsonOut.dumps(get_world(randint(1, 10000)))

def queries():
    response.headers["Content-Type"]="application/json"
    num_queries = getQueryNum(request.vars["queries"])
    rp = partial(randint, 1, 10000)
    worlds = [get_world(rp()) for _ in xrange(num_queries)]
    return jsonOut.dumps(worlds)

def updates():
    response.headers["Content-Type"]="application/json"
    num_queries = getQueryNum(request.vars["queries"])
    worlds = []
    rp = partial(randint, 1, 10000)
    ids = [rp() for _ in xrange(num_queries)]
    ids.sort() # To avoid deadlock
    for wid in ids:
        world = get_world(wid)
        newNumber = rp()
        db(db.world.id == wid).update(randomNumber=newNumber)
        world.update(randomNumber=newNumber)
        worlds.append(world)
    return jsonOut.dumps(worlds)

def fortune():
    fortunes = db(db.fortune).select(processor=processor)
    fortune_list.append({"id": 0,
                         "message": "Additional fortune added at request time."})
    fortune_list = sorted(fortune_list, key=lambda k: k["message"])
    return dict(fortunes=fortune_list)
